"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const where_builder_1 = require("./where.builder");
/**
 * @internal
 *
 * Class that will convert a Query into a `sequelize` Query Builder.
 */
class FilterQueryBuilder {
    constructor(whereBuilder = new where_builder_1.WhereBuilder()) {
        this.whereBuilder = whereBuilder;
    }
    /**
     * Create a `sequelize` SelectQueryBuilder with `WHERE`, `ORDER BY` and `LIMIT/OFFSET` clauses.
     *
     * @param query - the query to apply.
     */
    findOptions(query) {
        let opts = {};
        opts = this.applyFilter(opts, query.filter);
        opts = this.applySorting(opts, query.sorting);
        opts = this.applyPaging(opts, query.paging);
        return opts;
    }
    /**
     * Create a `sequelize` DeleteQueryBuilder with a WHERE clause.
     *
     * @param query - the query to apply.
     */
    destroyOptions(query) {
        let opts = {};
        opts = this.applyFilter(opts, query.filter);
        opts = this.applyPaging(opts, query.paging);
        return opts;
    }
    /**
     * Create a `sequelize` UpdateQueryBuilder with `WHERE` and `ORDER BY` clauses
     *
     * @param query - the query to apply.
     */
    updateOptions(query) {
        let opts = { where: {} };
        opts = this.applyFilter(opts, query.filter);
        opts = this.applyPaging(opts, query.paging);
        return opts;
    }
    /**
     * Applies paging to a Pageable `sequelize` query builder
     * @param qb - the `sequelize` QueryBuilder
     * @param paging - the Paging options.
     */
    applyPaging(qb, paging) {
        if (!paging) {
            return qb;
        }
        if (paging.limit !== undefined) {
            // eslint-disable-next-line no-param-reassign
            qb.limit = paging.limit;
        }
        if (paging.offset !== undefined) {
            // eslint-disable-next-line no-param-reassign
            qb.offset = paging.offset;
        }
        return qb;
    }
    /**
     * Applies the filter from a Query to a `sequelize` QueryBuilder.
     *
     * @param qb - the `sequelize` QueryBuilder.
     * @param filter - the filter.
     */
    applyFilter(filterable, filter) {
        if (!filter) {
            return filterable;
        }
        // eslint-disable-next-line no-param-reassign
        filterable.where = this.whereBuilder.build(filter);
        return filterable;
    }
    /**
     * Applies the ORDER BY clause to a `sequelize` QueryBuilder.
     * @param qb - the `sequelize` QueryBuilder.
     * @param sorts - an array of SortFields to create the ORDER BY clause.
     */
    applySorting(qb, sorts) {
        if (!sorts) {
            return qb;
        }
        // eslint-disable-next-line no-param-reassign
        qb.order = sorts.map(({ field, direction, nulls }) => {
            const col = `${field}`;
            const dir = [direction];
            if (nulls) {
                dir.push(nulls);
            }
            return [col, dir.join(' ')];
        });
        return qb;
    }
}
exports.FilterQueryBuilder = FilterQueryBuilder;
//# sourceMappingURL=filter-query.builder.js.map